""" Manages the generation of C++ header files """

from dune.perftool.generation import retrieve_cache_items

def generate_file(filename, tag, content, headerguard=True):
    """
    Write a file from the generation cache:

    Arguments:
    ----------

    Keyword Arguments:
    ------------------
    """
    with open(filename, 'w') as f:
        # Add a double inclusion protection header
        if headerguard:
            macro = filename.upper().replace("/", "_").replace(".", "_")
            f.write("#ifndef {0}\n#define {0}\n\n".format(macro))

        # Add the includes from the cache
        for inc in retrieve_cache_items((tag, 'include'), union=False):
            from cgen import Include
            assert isinstance(inc, Include)
            for line in inc.generate():
                f.write(line)
            f.write('\n')

        f.write('\n\n')

        for c in content:
            from cgen import Generable
            assert isinstance(c, Generable)
            for line in c.generate():
                f.write(line)

        if headerguard:
            f.write("\n\n#endif //{}".format(macro))
