from __future__ import absolute_import
from ufl.algorithms import MultiFunction
from dune.perftool.ufl.transformations import ufl_transformation

# TODO: This follows a pattern that may be worth abstracting!
# It replaces the ifinstance orgy mappers when having operand dependant behaviour.
class IndexedMapper(MultiFunction):
    
    call = MultiFunction.__call__
    
    def __init__(self, rim):
        MultiFunction.__init__(self)
        # The corresponding ReindexingMapper
        self.rim = rim
    
    def __call__(self, o):
        from ufl.classes import Indexed
        assert isinstance(o, Indexed)
        
        self.indexed_expr = o
        
        return self.call(o.ufl_operands[0])
    
    def expr(self, o):
        # If we are here, this one does not need special treatment, we use the other one
        # TODO: apply renaming!
        return self.reuse_if_untouched(self.indexed_expr, self.rim.call(o), self.rim.call(self.indexed_expr.ufl_operands[1]))
    
    def component_tensor(self, o):
        tensor, idx = o.ufl_operands
        
        # Extract an index mapping
        from itertools import izip
        for inner, outer in izip(idx, self.indexed_expr.ufl_operands[1]):
            self.rim.replacement_map[inner] = self.rim.replacement_map.setdefault(outer, outer)
            
        # And return an expression with the component tensor removed
        return self.rim.call(tensor)
        

class ReindexingMapper(MultiFunction):
    
    call = MultiFunction.__call__
    
    def __init__(self):
        MultiFunction.__init__(self)
        self.replacement_map = {}
        self.multi_index_cache = {}
        self.im = IndexedMapper(self)
    
    def expr(self, o):
        return self.reuse_if_untouched(o, *tuple(self.call(op) for op in o.ufl_operands))

    def indexed(self, o):
        return self.im(o)

    def multi_index(self, o):
        newindices = tuple(self.replacement_map.get(i, i) for i in o)
        from ufl.classes import MultiIndex
        return self.multi_index_cache.setdefault(newindices, MultiIndex(newindices))


@ufl_transformation(name="reindexing")
def reindexing(e):
    return ReindexingMapper()(e)

